
"""
This file is part of the SiEPIC_EBeam_PDK
by Lukas Chrostowski (c) 2015-2023

This Python file implements a library called "SiEPIC_EBeam", consisting of mature components that
have Layouts and Compact Models for circuit simulations:
 - Fixed GDS cell components: imported from SiEPIC-EBeam.gds
 - PCells:
    - ebeam_dc_halfring_straight
    - ebeam_bragg_te1550: waveguide Bragg grating
                - ebeam_taper_te1550: Waveguide Taper
                - Waveguide_bump
                -
                - Waveguide_Bend

NOTE: after changing the code, the macro needs to be rerun to install the new
implementation. The macro is also set to "auto run" to install the PCell
when KLayout is run.

Crash warning:
 https://www.klayout.de/forum/discussion/734
 This library has nested PCells. Running this macro with a layout open may
 cause it to crash. Close the layout first before running.

Version history:

Lukas Chrostowski           2015/11/05 - 2015/11/10
 - Double-bus ring resonator
 - waveguide bends
 - PCell parameter functions
 - polygon text
 - PCell calling another PCell - TestStruct_DoubleBus_Ring

Lukas Chrostowski           2015/11/14
 - fix for rounding error in "DoubleBus_Ring"

Lukas Chrostowski           2015/11/15
 - fix for Python 3.4: print("xxx")

Lukas Chrostowski           2015/11/17
 - update "layout_waveguide_rel" to use the calculated points_per_circle(radius)

Lukas Chrostowski           2015/11/xx
 - Waveguide based on bends, straight waveguide.

Lukas Chrostowski           2015/12/3
 - Bragg grating

Lukas Chrostowski           2016/01/17
 - Taper, matching EBeam CML component

Lukas Chrostowski           2016/01/20
 - (sinusoidal) Bragg grating

Lukas Chrostowski           2016/05/27
 - SWG_waveguide
 - SWG_to_strip_waveguide

S. Preble                   2016/08/26
 - Double Bus Ring Pin's shifted - text now is in the middle of the pin path

Lukas Chrostowski           2016/11/06
 - waveguide bump, to provide a tiny path length increase

Lukas Chrostowski           2017/02/14
 - renaming "SiEPIC" PCells library to "SiEPIC-EBeam PCells", update for Waveguide_Route
 - code simplifications: Box -> Box

Lukas Chrostowski           2017/03/08
 - S-Bend -

Lukas Chrostowski           2017/03/18
 - ebeam_dc_halfring_straight, with TE/TM support.  Zeqin Lu adding CML for this component.

Lukas Chrostowski 2017/12/16
 - compatibility with KLayout 0.25 and SiEPIC-Tools

Lukas Chrostowski 2020/02/22
 - contra directional coupler; moved from Dev library; now has a compact model generator

todo:
replace:
 layout_arc_wg_dbu(self.cell, Layerm1N, x0,y0, r_m1_in, w_m1_in, angle_min_doping, angle_max_doping)
with:
 self.cell.shapes(Layerm1N).insert(Polygon(arc(w_m1_in, angle_min_doping, angle_max_doping) +
 [Point(0, 0)]).transformed(t))


Lukas Chrostowski 2020/06/21
 - Waveguide update, cellName

Mustafa Hammood 2020/06/26
 - major refactoring and splitting of individual classes into sub files

Lukas 2021/04/01
 - fixing loading library (previous collisions with other PDKs)

Lukas 2023/11
 - compatibility with PyPI usage of KLayout
 
Lukas 2024/10
 - moving most of the code into SiEPIC.scripts.load_klayout_library

"""



import os
import sys
import SiEPIC

from SiEPIC.utils import get_technology_by_name



"""
This file is part of the SiEPIC_EBeam_PDK
by Dream Photonics, 2023

This file implements a library called "EBeam-Dream",
consisting of components developed by Dream Photonics Inc,
that have been validated in the ANT process.
"""




"""
This file is part of the SiEPIC_EBeam_PDK
by Lukas Chrostowski, et al., (c) 2023

This file implements a library called "EBeam-SiN", for silicon nitride


Version history:

Lukas Chrostowski, 2023/01/24
 - Creation of a new library for Silicon Nitride devices
 - directional coupler PCell for 895 nm
 - Added SiN waveguide to WAVEGUIDES.XML

Lukas Chrostowski, 2023/11
 - components for 1550 and 1310
 - waveguides for 1550 and 1310
 - compound waveguide
 - compatibility with PyPI usage of KLayout

"""



print("SiEPIC-EBeam-PDK Python module: siepic_ebeam_pdk, KLayout technology: EBeam")

# Check KLayout version
import SiEPIC
from SiEPIC._globals import KLAYOUT_VERSION
if KLAYOUT_VERSION < 28:
    question = pya.QMessageBox()
    question.setStandardButtons(pya.QMessageBox.Ok)
    question.setText(
        "SiEPIC-EBeam is not compatible with older versions (<0.28) of KLayout."
    )
    KLayout_link0 = "https://www.klayout.de/build.html"
    question.setInformativeText(
        "\nSiEPIC-EBeam is not compatible with older versions (<0.28) of KLayout.\nPlease download an install the latest version, from %s"
        % (KLayout_link0)
    )
    pya.QMessageBox_StandardButton(question.exec_())

from packaging.version import Version
if Version(SiEPIC.__version__) < Version('0.5.14'):
    raise Exception ('This PDK requires SiEPIC-Tools v0.5.14 or greater.')

# Load the KLayout technology, when running in Script mode
import pya
import os

if not pya.Technology().has_technology("EBeam"):
    tech = pya.Technology().create_technology("EBeam")
    tech.load(os.path.join(os.path.dirname(os.path.realpath(__file__)), "EBeam.lyt"))

# then import all the technology modules
from . import pymacros

# List the libraries loaded       
from SiEPIC.scripts import technology_libraries
technology_libraries(tech)




# $autorun

version = "0.4.24"

print('SiEPIC-EBeam-PDK Python module: pymacros')

from SiEPIC.scripts import load_klayout_library

verbose=False

tech='EBeam'

# Load the library
load_klayout_library(tech, 'EBeam', "v%s, Components with models" % version, 'gds/EBeam','pymacros/pcells_EBeam', verbose=verbose)
load_klayout_library(tech, 'EBeam_Beta', "v%s, Beta components" % version, 'gds/EBeam_Beta','pymacros/pcells_EBeam_Beta', verbose=verbose)
load_klayout_library(tech, 'EBeam-Dream', "v%s, Dream Photonics" % version, 'gds/EBeam_Dream','pymacros/pcells_EBeam_Dream', verbose=verbose)
load_klayout_library(tech, 'EBeam-SiN', "v%s, Silicon Nitride" % version, 'gds/EBeam_SiN','pymacros/pcells_SiN', verbose=verbose)
load_klayout_library(tech, 'EBeam-ANT', "v%s, ANT components" % version, 'gds/ANT','', verbose=verbose)


